package com.blotout.blotoutfluttersdk;

import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;

import com.blotout.analytics.BlotoutAnalytics;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * BlotoutfluttersdkPlugin
 */
public class BlotoutfluttersdkPlugin implements FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;
    private static final String name = "blotoutfluttersdk";
    private Context context;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), BlotoutfluttersdkPlugin.name);
        context = flutterPluginBinding.getApplicationContext();
        channel.setMethodCallHandler(this);
    }

    // This static voidtion is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this voidtion while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them voidtionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "blotoutfluttersdk");
        channel.setMethodCallHandler(new BlotoutfluttersdkPlugin());
    }

    public BlotoutfluttersdkPlugin() {
    }

    public BlotoutfluttersdkPlugin(Context context) {
        this.context = context;
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case BlotoutFlutterConstants.Initialize_SDK:
                this.initializeAnalyticsEngine(call, result);
                break;
            case BlotoutFlutterConstants.Update_Blotout_SDK:
                this.updateAnalyticsEngineTest(call, result);
                break;
            case BlotoutFlutterConstants.Start_Timed_Event:
                this.startTimedEvent(call, result);
                break;
            case BlotoutFlutterConstants.End_Timed_Event:
                this.endTimedEvent(call, result);
                break;
            case BlotoutFlutterConstants.Log_Event:
                this.logEvent(call, result);
                break;
            case BlotoutFlutterConstants.Log_Event_With_Time:
                this.logEventWithTime(call, result);
                break;
            case BlotoutFlutterConstants.Log_PII_Event:
                this.logPIIEvent(call, result);
                break;
            case BlotoutFlutterConstants.Log_PHI_Event:
                this.logPHIEvent(call, result);
                break;
            case BlotoutFlutterConstants.Log_User_Retention_Event:
                this.logUserRetentionEvent(call, result);
                break;
            case BlotoutFlutterConstants.Set_SDK_End_Point_Url:
                this.setSDKEndPoint(call, result);
                break;
            case BlotoutFlutterConstants.Set_SDK_Enabled:
                this.setSDKEnabled(call, result);
                break;
            case BlotoutFlutterConstants.Set_Network_Sync_Enabled:
                this.setNetworkSyncEnabled(call, result);
                break;
            case BlotoutFlutterConstants.Set_Funnel_Events_Enabled:
                this.setFunnelEventsEnabled(call, result);
                break;
            case BlotoutFlutterConstants.Set_Segment_Events_Enabled:
                this.setSegmentEventsEnabled(call, result);
                break;
            case BlotoutFlutterConstants.Set_Data_Collection_Enabled:
                this.setDataCollectionEnabled(call, result);
                break;
            case BlotoutFlutterConstants.Set_System_Events_Enabled:
                this.setSystemEventsEnabled(call, result);
                break;
            case BlotoutFlutterConstants.Set_Paying_User:
                this.setPayingUser(call, result);
                break;
            case BlotoutFlutterConstants.Set_Developer_Events_Enabled:
                this.setDeveloperEventsEnabled(call, result);
                break;
            case BlotoutFlutterConstants.Set_Retention_Events_Enabled:
                this.setRetentionEventsEnabled(call, result);
                break;

            case BlotoutFlutterConstants.IS_Device_Compromised:
                this.isDeviceCompromised(call, result);
                break;

            case BlotoutFlutterConstants.IS_App_Compromised:
                this.isAppCompromised(call, result);
                break;

            case BlotoutFlutterConstants.IS_Simulator:
                this.isSimulator(call, result);
                break;

            case BlotoutFlutterConstants.IS_Environment_Secure:
                this.isEnvironmentSecure(call, result);
                break;

            case BlotoutFlutterConstants.IS_Network_Proxied:
                this.isNetworkProxied(call, result);
                break;

            case BlotoutFlutterConstants.IS_Running_On_VM:
                this.isRunningOnVM(call, result);
                break;

            default:
                break;
        }
    }

    private void initializeAnalyticsEngine(MethodCall call, Result result) {

        final String prodKey = call.argument("prodKey");
        final String testKey = call.argument("testKey");
        if (prodKey == null || testKey == null) {
            Log.e("BlotoutfluttersdkPlugin", "Your BlotoutAnalytics Token was not informed");
        }

        Boolean isProdMode = call.<Boolean>argument("isProdMode");
        BlotoutAnalytics.getInstance().initializeAnalyticsEngine(this.context, testKey, prodKey, isProdMode == null ? false : isProdMode);

        result.success(true);
    }


    /*
     * @param blotoutTestKey stage key as String
     * @param blotoutProductionKey production key as String
     */
    private void updateAnalyticsEngineTest(MethodCall call, Result result) {
        final String prodKey = call.argument("prodKey");
        final String testKey = call.argument("testKey");
        if (prodKey == null || testKey == null) {
            Log.e("BlotoutfluttersdkPlugin", "Your BlotoutAnalytics Token was not informed");
        }

        BlotoutAnalytics.getInstance().updateAnalyticsEngine(this.context, testKey, prodKey);

    }

    /*
     * @param eventName name of the event
     * @param startEventInfo properties in key/value pair
     */
    private void startTimedEvent(MethodCall call, Result result) {
        String eventName = call.argument("eventName");
        HashMap<String, Object> mapProperties = call.<HashMap<String, Object>>argument("properties");
        BlotoutAnalytics.getInstance().startTimedEvent(eventName, mapProperties);
    }

    /*
     * @param eventName name of the event
     * @param endEventInfo properties in key/value pair
     */
    private void endTimedEvent(MethodCall call, Result result) {
        String eventName = call.argument("eventName");
        HashMap<String, Object> mapProperties = call.<HashMap<String, Object>>argument("properties");
        BlotoutAnalytics.getInstance().endTimedEvent(eventName, mapProperties);
    }

    /*
     * @param eventName name of the event
     * @param eventInfo properties in key/value pair
     */
    private void logEvent(MethodCall call, Result result) {
        String eventName = call.argument("eventName");
        HashMap<String, Object> mapProperties = call.<HashMap<String, Object>>argument("properties");
        BlotoutAnalytics.getInstance().logEvent(eventName, mapProperties);
    }

    /*
     * @param eventName name of the event as String
     * @param eventInfo properties in key/value pair
     * @param eventTime eventTime as Date
     */
    private void logEventWithTime(MethodCall call, Result result) {
        String eventName = call.argument("eventName");
        HashMap<String, Object> mapProperties = call.<HashMap<String, Object>>argument("properties");
        Long dateTimeStamp = call.<Long>argument("time");
        Date dateTime = new Date(dateTimeStamp);
        BlotoutAnalytics.getInstance().logEvent(eventName, mapProperties, dateTime);
    }

    /*
     * @param eventName name of the event as String
     * @param eventInfo properties in key/value pair
     */
    private void logUserRetentionEvent(MethodCall call, Result result) {
        String eventName = call.argument("eventName");
        HashMap<String, Object> mapProperties = call.<HashMap<String, Object>>argument("properties");
        BlotoutAnalytics.getInstance().logUserRetentionEvent(eventName, mapProperties);
    }

    /*
     * @param eventName name of the event as String
     * @param eventInfo properties in key/value pair
     */
    private void logPIIEvent(MethodCall call, Result result) {
        String eventName = call.argument("eventName");
        HashMap<String, Object> mapProperties = call.<HashMap<String, Object>>argument("properties");
        Long dateTimeStamp = call.<Long>argument("time");
        Date dateTime = new Date(dateTimeStamp);
        BlotoutAnalytics.getInstance().logPIIEvent(eventName, mapProperties, dateTime);
    }

    /*
     * @param eventName name of the event as String
     * @param eventInfo properties in key/value pair
     */
    private void logPHIEvent(MethodCall call, Result result) {
        String eventName = call.argument("eventName");
        HashMap<String, Object> mapProperties = call.<HashMap<String, Object>>argument("properties");
        Long dateTimeStamp = call.<Long>argument("time");
        Date dateTime = new Date(dateTimeStamp);
        BlotoutAnalytics.getInstance().logPHIEvent(eventName, mapProperties, dateTime);
    }

    /**
     * SDK Endpoint Url must be set before Analytics Engine Initialization, It has to be a domain based IP's are considered invalid.
     *
     * @param
     */
    private void setSDKEndPoint(MethodCall call, Result result) {
        String endPoint = call.argument("endPoint");
        BlotoutAnalytics.getInstance().setSDKEndPointUrl(endPoint);
    }


    private void setSDKEnabled(MethodCall call, Result result) {
        Boolean sdkEnabled = call.<Boolean>argument("sdkEnabled");
        BlotoutAnalytics.getInstance().setEnabled(sdkEnabled);
    }

    /*
     * @param networkSyncEnabled default is true, set false to disable the Network call
     */

    private void setNetworkSyncEnabled(MethodCall call, Result result) {
        Boolean enabled = call.<Boolean>argument("enabled");
        BlotoutAnalytics.getInstance().setNetworkSyncEnabled(enabled);
    }

    private void setDataCollectionEnabled(MethodCall call, Result result) {
        Boolean enabled = call.<Boolean>argument("enabled");
        BlotoutAnalytics.getInstance().setDataCollectionEnabled(enabled);
    }

    private void setFunnelEventsEnabled(MethodCall call, Result result) {
        Boolean enabled = call.<Boolean>argument("enabled");
        BlotoutAnalytics.getInstance().setFunnelEventsEnabled(enabled);
    }

    private void setSegmentEventsEnabled(MethodCall call, Result result) {
        Boolean enabled = call.<Boolean>argument("enabled");
        BlotoutAnalytics.getInstance().setSegmentEventsEnabled(enabled);
    }

    private void setRetentionEventsEnabled(MethodCall call, Result result) {
        Boolean enabled = call.<Boolean>argument("enabled");
        BlotoutAnalytics.getInstance().setRetentionEventsEnabled(enabled);
    }

    private void setDeveloperEventsEnabled(MethodCall call, Result result) {
        Boolean enabled = call.<Boolean>argument("enabled");
        BlotoutAnalytics.getInstance().setDeveloperEventsEnabled(enabled);
    }

    private void setSystemEventsEnabled(MethodCall call, Result result) {
        Boolean enabled = call.<Boolean>argument("enabled");
        BlotoutAnalytics.getInstance().setSystemEventsEnabled(enabled);
    }

    private void setPayingUser(MethodCall call, Result result) {
        Boolean enabled = call.<Boolean>argument("enabled");
        BlotoutAnalytics.getInstance().setPayingUser(enabled);
    }

    private void isDeviceCompromised(MethodCall call, Result result) {
        result.success(BlotoutAnalytics.getInstance().isDeviceCompromised());
    }

    private void isAppCompromised(MethodCall call, Result result) {
        result.success(BlotoutAnalytics.getInstance().isAppCompromised());
    }

    private void isNetworkProxied(MethodCall call, Result result) {
        result.success(BlotoutAnalytics.getInstance().isNetworkProxied());
    }

    private void isSimulator(MethodCall call, Result result) {
        result.success(BlotoutAnalytics.getInstance().isDeviceCompromised());
    }

    private void isRunningOnVM(MethodCall call, Result result) {
        result.success(BlotoutAnalytics.getInstance().isRunningOnVM());
    }

    private void isEnvironmentSecure(MethodCall call, Result result) {
        result.success(BlotoutAnalytics.getInstance().isEnvironmentSecure());
    }

}

class BlotoutFlutterConstants {

    public static final String Initialize_SDK = "initBlotoutSDK";
    public static final String Update_Blotout_SDK = "updateBlotoutSDK";
    public static final String Start_Timed_Event = "startTimedEvent";
    public static final String End_Timed_Event = "endTimedEvent";
    public static final String Log_Event = "logEvent";
    public static final String Log_Event_With_Time = "logEventWithTime";
    public static final String Log_User_Retention_Event = "logUserRetentionEvent";
    public static final String Log_PII_Event = "logPIIEvent";
    public static final String Log_PHI_Event = "logPHIEvent";

    public static final String Set_SDK_End_Point_Url = "setSDKEndPointUrl";
    public static final String Set_SDK_Enabled = "setSDKEnabled";
    public static final String Set_Network_Sync_Enabled = "setNetworkSyncEnabled";
    public static final String Set_Data_Collection_Enabled = "setDataCollectionEnabled";
    public static final String Set_Funnel_Events_Enabled = "setFunnelEventsEnabled";
    public static final String Set_Segment_Events_Enabled = "setSegmentEventsEnabled";
    public static final String Set_System_Events_Enabled = "setSystemEventsEnabled";
    public static final String Set_Developer_Events_Enabled = "setDeveloperEventsEnabled";
    public static final String Set_Paying_User = "setPayingUser";
    public static final String Set_Retention_Events_Enabled = "setRetentionEventsEnabled";

    public static final String IS_Device_Compromised = "isDeviceCompromised";
    public static final String IS_App_Compromised = "isAppCompromised";
    public static final String IS_Network_Proxied = "isNetworkProxied";
    public static final String IS_Simulator = "isSimulator";
    public static final String IS_Running_On_VM = "isRunningOnVM";
    public static final String IS_Environment_Secure = "isEnvironmentSecure";
}

